@@ -84,51 +84,6 @@ class nlmsg_base(dict):
     you're inventing completely new protocol structure.
 
     Use nlmsg or nla classes.
-
-    ...
-
-    Netlink message structure
-
-    | header | data |
-
-    nlmsg header:
-        + uint32 length
-        + uint16 type
-        + uint16 flags
-        + uint32 sequence number
-        + uint32 pid
-
-    nla header:
-        + uint16 length
-        + uint16 type
-
-    data:
-        + data-specific struct
-        + NLA
-        + NLA
-        + ...
-
-    To describe data, use python struct notation:
-
-        fields = (('length', 'H'),
-                  ('type', 'H'))
-
-
-    NLAs are decoded/encoded according to 'nla_map':
-
-        nla_map = [['NDA_UNSPEC', 'none'],
-                   ['NDA_DST', 'ipaddr'],
-                   ['NDA_LLADDR', 'l2addr'],
-                   ['NDA_CACHEINFO', 'cacheinfo'],
-                   ['NDA_PROBES', 'uint32']]
-
-    Please note, that 'nla_map' creates implied enumeration from
-    its fields. In the example above NDA_UNSPEC == 0 and
-    NDA_PROBES == 4. These numbers will be used as uint16 'type'
-    in NLA header.
-
-    List of public types as 'none', 'uint32', 'ipaddr', 'asciiz'
-    you can read in nlmsg_atoms class.
     '''
 
     fields = []                  # data field names, to build a dictionary
@@ -501,13 +456,21 @@ class nlmsg_base(dict):
         self.t_nla_map = {}
         self.r_nla_map = {}
 
-        # create enumeration
-        nla_types = enumerate((i[0] for i in self.nla_map))
-        # that's a little bit tricky, but to reduce
-        # the required amount of code in modules, we have
-        # to jump over the head
-        zipped = [(k[1][0], k[0][0], k[0][1]) for k in
-                  zip(self.nla_map, nla_types)]
+        # work only on non-empty mappings
+        if not self.nla_map:
+            return
+
+        # detect, whether we have pre-defined keys
+        if len(self.nla_map[0]) == 2:
+            # create enumeration
+            nla_types = enumerate((i[0] for i in self.nla_map))
+            # that's a little bit tricky, but to reduce
+            # the required amount of code in modules, we have
+            # to jump over the head
+            zipped = [(k[1][0], k[0][0], k[0][1]) for k in
+                      zip(self.nla_map, nla_types)]
+        else:
+            zipped = self.nla_map
 
         for (key, name, nla_class) in zipped:
             # lookup NLA class
@@ -542,6 +505,7 @@ class nlmsg_base(dict):
     def decode_nlas(self):
         while self.buf.tell() < (self.offset + self.length):
             init = self.buf.tell()
+            nla = None
             # pick the length and the type
             (length, msg_type) = struct.unpack('HH', self.buf.read(4))
             # rewind to the beginning
@@ -563,24 +527,25 @@ class nlmsg_base(dict):
                 # decode NLA
                 nla = msg_class(self.buf, length, self,
                                 debug=self.debug)
-                try:
-                    nla.decode()
-                except:
-                    # FIXME
-                    self.buf.seek(init)
-                    msg_value = hexdump(self.buf.read(length))
-                else:
-                    msg_value = nla.getvalue()
+            try:
+                nla.decode()
+            except:
+                # FIXME
+                self.buf.seek(init)
+                msg_name = 'UNKNOWN'
+                msg_value = hexdump(self.buf.read(length))
+            else:
+                msg_value = nla.getvalue()
 
-                if self.debug:
-                    self['attrs'].append([msg_name,
-                                          msg_value,
-                                          msg_type,
-                                          length,
-                                          init])
-                else:
-                    self['attrs'].append([msg_name,
-                                          msg_value])
+            if self.debug:
+                self['attrs'].append([msg_name,
+                                      msg_value,
+                                      msg_type,
+                                      length,
+                                      init])
+            else:
+                self['attrs'].append([msg_name,
+                                      msg_value])
 
             # fix the offset
             self.buf.seek(init + NLMSG_ALIGN(length))
