@@ -226,6 +226,10 @@ class CommitException(Exception):
     pass
 
 
+class CreateException(Exception):
+    pass
+
+
 def clear_fail_bit(bit):
     global _FAIL_MASK
     _FAIL_MASK &= ~(_FAIL_MASK & bit)
@@ -317,6 +321,7 @@ class LinkedSet(set):
         self._ct = None
         self.raw = {}
         self.links = []
+        self.exclusive = set()
 
     def set_target(self, value):
         '''
@@ -348,7 +353,7 @@ class LinkedSet(set):
                     self._ct = None
                     self.target.set()
 
-    def add(self, key, raw=None):
+    def add(self, key, raw=None, cascade=False):
         '''
         Add an item to the set and all connected instances,
         check the target state.
@@ -362,25 +367,41 @@ class LinkedSet(set):
         human-readable ip addr representation.
         '''
         with self.lock:
+            if cascade and (key in self.exclusive):
+                return
             if key not in self:
                 self.raw[key] = raw
                 set.add(self, key)
                 for link in self.links:
-                    link.add(key, raw)
+                    link.add(key, raw, cascade=True)
             self.check_target()
 
-    def remove(self, key, raw=None):
+    def remove(self, key, raw=None, cascade=False):
         '''
         Remove an item from the set and all connected instances,
         check the target state.
         '''
         with self.lock:
+            if cascade and (key in self.exclusive):
+                return
             set.remove(self, key)
             for link in self.links:
                 if key in link:
-                    link.remove(key)
+                    link.remove(key, cascade=True)
             self.check_target()
 
+    def unlink(self, key):
+        '''
+        Exclude key from cascade updates.
+        '''
+        self.exclusive.add(key)
+
+    def relink(self, key):
+        '''
+        Do not ignore key on cascade updates.
+        '''
+        self.exclusive.remove(key)
+
     def connect(self, link):
         '''
         Connect a LinkedSet instance to this one. Connected
@@ -553,6 +574,7 @@ class Transactional(Dotkeys):
         self._tids = []
         self._transactions = {}
         self._targets = {}
+        self._local_targets = {}
         self._mode = mode or ipdb.mode
         self._write_lock = threading.RLock()
         self._direct_state = State(self._write_lock)
@@ -676,19 +698,34 @@ class Transactional(Dotkeys):
     @update
     def __setitem__(self, direct, key, value):
         if not direct:
+            # automatically set target on the last transaction,
+            # which must be started prior to that call
             transaction = self.last()
             transaction[key] = value
             transaction._targets[key] = threading.Event()
         else:
+            # update on local targets
+            if key in self._local_targets:
+                func = self._fields_cmp.get(key, lambda x, y: x == y)
+                if func(value, self._local_targets[key].value):
+                    self._local_targets[key].set()
+
+            # cascade update on nested targets
             for tn in self._transactions.values():
                 if key in tn._targets:
                     if self._fields_cmp.\
                             get(key, lambda x, y: x == y)(value, tn[key]):
                         tn._targets[key].set()
+
+            # and, finally, set the item :)
             Dotkeys.__setitem__(self, key, value)
 
     @update
     def __delitem__(self, direct, key):
+        # firstly set targets
+        self[key] = None
+
+        # then continue with delete
         if not direct:
             transaction = self.last()
             if key in transaction:
@@ -704,6 +741,13 @@ class Transactional(Dotkeys):
         del self[key]
         return self
 
+    def set_target(self, key, value):
+        self._local_targets[key] = threading.Event()
+        self._local_targets[key].value = value
+
+    def mirror_target(self, key_from, key_to):
+        self._local_targets[key_to] = self._local_targets[key_from]
+
     def set_item(self, key, value):
         with self._direct_state:
             self[key] = value
@@ -871,6 +915,7 @@ class Interface(Transactional):
             transaction = self.last()
             transaction.add_ip(ip, mask)
         else:
+            self['ipaddr'].unlink((ip, mask))
             self['ipaddr'].add((ip, mask))
         return self
 
@@ -890,6 +935,7 @@ class Interface(Transactional):
             if (ip, mask) in transaction['ipaddr']:
                 transaction.del_ip(ip, mask)
         else:
+            self['ipaddr'].unlink((ip, mask))
             self['ipaddr'].remove((ip, mask))
         return self
 
@@ -904,6 +950,7 @@ class Interface(Transactional):
             transaction = self.last()
             transaction.add_port(port)
         else:
+            self['ports'].unlink(port)
             compat.fix_add_master(self.ipdb.interfaces[port], self)
             self['ports'].add(port)
         return self
@@ -920,6 +967,7 @@ class Interface(Transactional):
             if port in transaction['ports']:
                 transaction.del_port(port)
         else:
+            self['ports'].unlink(port)
             compat.fix_del_master(self.ipdb.interfaces[port])
             # FIXME: do something with it, please
             self['ports'].remove(port)
@@ -1044,17 +1092,36 @@ class Interface(Transactional):
 
                 for i in removed['ports']:
                     # detach the port
-                    compat.fix_del_port(self.nl, self, self.ipdb.interfaces[i])
+                    port = self.ipdb.interfaces[i]
+                    port.set_target('master', None)
+                    port.mirror_target('master', 'link')
+                    compat.fix_del_port(self.nl, self, port)
 
                 for i in added['ports']:
                     # enslave the port
-                    compat.fix_add_port(self.nl, self, self.ipdb.interfaces[i])
+                    port = self.ipdb.interfaces[i]
+                    port.set_target('master', self['index'])
+                    port.mirror_target('master', 'link')
+                    compat.fix_add_port(self.nl, self, port)
 
                 if removed['ports'] or added['ports']:
                     self.nl.get_links(*(removed['ports'] | added['ports']))
                     self['ports'].target.wait(_SYNC_TIMEOUT)
                     if not self['ports'].target.is_set():
                         raise CommitException('ports target is not set')
+                    # wait for proper targets on ports
+                    for i in list(added['ports']) + list(removed['ports']):
+                        port = self.ipdb.interfaces[i]
+                        target = port._local_targets['master']
+                        target.wait(_SYNC_TIMEOUT)
+                        del port._local_targets['master']
+                        del port._local_targets['link']
+                        if not target.is_set():
+                            raise CommitException('master target failed')
+                        if i in added['ports']:
+                            assert port.if_master == self['index']
+                        else:
+                            assert port.if_master != self['index']
 
                 # 8<---------------------------------------------
                 # Interface changes
@@ -1556,7 +1623,7 @@ class IPDB(object):
             self.nl.release()
             self._mthread.join()
 
-    def create(self, kind, ifname, **kwarg):
+    def create(self, kind, ifname, reuse=False, **kwarg):
         '''
         Create an interface. Arguments 'kind' and 'ifname' are
         required.
@@ -1568,6 +1635,7 @@ class IPDB(object):
           * tun
           * dummy
         * ifname -- interface name
+        * reuse -- if such interface exists, return it anyway
 
         Different interface kinds can require different
         arguments for creation.
@@ -1576,10 +1644,13 @@ class IPDB(object):
         '''
         with self.exclusive:
             # check for existing interface
-            if ((ifname in self.interfaces) and
-                    (self.interfaces[ifname]._flicker)):
-                device = self.interfaces[ifname]
-                device._flicker = False
+            if ifname in self.interfaces:
+                if self.interfaces[ifname]._flicker or reuse:
+                    device = self.interfaces[ifname]
+                    device._flicker = False
+                else:
+                    raise CreateException("interface %s exists" %
+                                          ifname)
             else:
                 device = \
                     self.by_name[ifname] = \
